package actionPilot;

import kinematics.Dynamics;
import kinematics.NewPose;
import kinematics.Pose2D;



/** The Arc move lets to robot travel in an arc. It does not alter the orientation of the robot.
 * @author Aswin
 *
 */
public class Arc extends AbstractMove {
	private float	radius;
	private float	radPerSecond;
	private float	startAngle=0;
	private float	direction;
	private Pose2D centre;

	
	/**
	 * @param centre
	 * A {@link Pose2D} object that represents the center of the Arc.
	 * @param angle
	 * The angle of the Arc to travel in radians
	 * @param direction
	 * The direction to travel. 1 = counterclockwise, -1 = clockwise.
	 */
	public Arc( Pose2D centre, float angle, int direction) {
    this.centre=new NewPose(centre.getX(), centre.getY(), angle);
		this.direction=Math.signum(direction);
	}
	
	public void setDynamics(Dynamics dynamics, Pose2D currentPose) {
	  super.setDynamics(dynamics, currentPose);
		radius = centre.length();
		startAngle = centre.directionTo(currentPose);
		radPerSecond = dynamics.getLinearSpeed() / radius ;
		actionTime = Math.abs(centre.getZ() / radPerSecond);
	}

	public boolean setTime( float time) {
	  if (!super.setTime(time)) return false;
		float a = (time * radPerSecond + startAngle);
		x=((float) (-Math.sin(a) * dynamics.getLinearSpeed()));
		y=( direction * (float) (Math.cos(a) * dynamics.getLinearSpeed()));
		return true;
	}



	@Override
	public String toString() {
		return "Arc";
	}

  @Override
  public boolean supportsDimension(int dimension) {
    if (dimension==2) return false;
    return true;
  }

}
