package kinematics;

/**
 * The Driver interface links a pilot to a chassis. 
 * The pilot determines the speed of a robot at any time and passes this on to a {@link Chassis} via the {{@link #move(Pose2D)} method this interface.
 * @author Aswin
 *
 */
public interface Driver {

  /**
   * Makes the motors float
   */
  public abstract void flt();

  /**
   * Makes the motors float
   */
  public abstract void flt(boolean immediateReturn);

  /**
   * Indicates whether the chassis is moving
   * @return
   * true when moving
   */
  public abstract boolean isMoving();

  /**
   * Instructs the chassis to stop
   */
  public abstract void stop();

  /**
   * Stops the chassis
   * @param immediateReturn
   * when false the method waits untill the chassis has stopped
   */
  public abstract void stop(boolean immediateReturn);

  /**
   * Instructs the Driver to moves at the speed specified
   * @param speed
   * The speed to move at. 
   */
  public abstract void move(Pose2D speed);

 
}