package kinematics;

import lejos.utility.Delay;

/** The Timing class helps to run a process at a target interval. 
 * It provides information on the exact length of the interval.<p>
 * See the {@link #delay()} method for additional details.
 * @author Aswin
 *
 */
public class Timing extends Delay {
  static final double factor=Math.pow(10, 9);
  protected long startTime=System.nanoTime();
  protected long lastTime=System.nanoTime();
  protected long interval=0;
  
  
  
  /** Constructer that sets the target interval
   * @param interval
   * The interval, expressed in seconds
   */
  public Timing(double interval) {
    setInterval(interval);
  }
  
  
  public Timing() {
  }


  /** Sets the target interval for the {@link #delay()} method.
   * @param interval
   * The interval, expressed in seconds
   */
  public void setInterval(double interval) {
    this.interval=(long) (interval*factor);
  }
  
  public float getInterval() {
    return (float) (interval/factor);
  }
  
  public void start() {
    startTime=System.nanoTime();
  }
  
  public void start(float when) {
    startTime=System.nanoTime()-(long)(when*factor);
  }
  
  public float elapsed() {
    return (float) ((System.nanoTime()-startTime)/factor);
  }
  
  
  /** The delay waits untill the target interval has elapsed.<p>
   *  The interval is measured from the previous time the method was called. 
   * @return
   * The exact length of the interval, expressed in seconds. 
   * The exact length of the interval is not the same as the target interval. It can be longer then the target interval 
   * if the time between two subsequent calls to this method exceeds the target interval.
   */
  public float delay() {
    long toWait=lastTime+interval-System.nanoTime();
    if (toWait>0)
      Delay.nsDelay(toWait);
    float dif=(float) ((System.nanoTime()-lastTime)/factor);
    lastTime=System.nanoTime();
    return dif;
  }

}
