package kinematics;

/** A basic vector class that holds three elements of type float and implements basic arithmatic.
 * @author Aswin
 *
 */
public class Vector implements Vector3D {
  protected float        x, y, z;

  public Vector() {

  }

  public Vector(float x, float y, float z) {
    this.x = x;
    this.y = y;
    this.z = z;
  }

  public Vector(Vector source) {
    x = source.x;
    y = source.y;
    z = source.z;
  }

  public String toString() {
    return "(" + x + ", " + y + ", " + z + ")";
  }

  @Override
  public float get(int index) {
    switch (index) {
      case 0:
        return x;
      case 1:
        return y;
      case 2:
        return z;
      default:
        throw new IndexOutOfBoundsException();
    }
  }

  public void set(int index, float value) {
    switch (index) {
      case 0:
        x = value;
        break;
      case 1:
        y = value;
        break;
      case 2:
        z = value;
        break;
      default:
        throw new IndexOutOfBoundsException();
    }
  }

  @Override
  public float getX() {
    return x;
  }

  public void setX(float x) {
    this.x = x;
  }

  @Override
  public float getY() {
    return y;
  }

  public void setY(float y) {
    this.y = y;
  }

  @Override
  public float getZ() {
    return z;
  }

  public void setZ(float z) {
    this.z = z;
  }

  public void set(float x, float y, float z) {
    this.x = x;
    this.y = y;
    this.z = z;
  }

  public void set(Vector3D vector) {
    x = vector.getX();
    y = vector.getY();
    z = vector.getZ();
  }

  public Vector plus(float value) {
    x += value;
    y += value;
    z += value;
    return this;
  }

  public Vector plus(Vector3D vector) {
    x += vector.getX();
    y += vector.getY();
    z += vector.getZ();
    return this;
  }

  public Vector minus(float value) {
    x -= value;
    y -= value;
    z -= value;
    return this;
  }

  public Vector minus(Vector3D vector) {
    x -= vector.getX();
    y -= vector.getY();
    z -= vector.getZ();
    return this;
  }

  public Vector times(float value) {
    x *= value;
    y *= value;
    z *= value;
    return this;
  }

  public Vector times(Vector3D vector) {
    x *= vector.getX();
    y *= vector.getY();
    z *= vector.getZ();
    return this;
  }
  
  public void times(Matrix m) {
    float xt,yt,zt;
    xt=x*m.get(0, 0)+y*m.get(0, 1)+z*m.get(0, 2);
    yt=x*m.get(1, 0)+y*m.get(1, 1)+z*m.get(1, 2);
    zt=x*m.get(2, 0)+y*m.get(2, 1)+z*m.get(2, 2);
    x= xt;
    y= yt;
    z=zt;
  }
  

  public Vector devide(float value) {
    x /= value;
    y /= value;
    z /= value;
    return this;
  }

  public Vector devide(Vector3D vector) {
    x /= vector.getX();
    y /= vector.getY();
    z /= vector.getZ();
    return this;
  }

}
