package samples;

import lejos.robotics.SampleProvider;
import lejos.robotics.filter.AbstractFilter;


/**
 * The errorTermnFilter calculates the difference between the actual sample and its moving average. 
  
 * @author Aswin
 *
 */
public class ErrorTermFilter extends AbstractFilter {
  static int SETTLE=10;
	float speed=0, endSpeed=0;
	float[] offset; 
	float error;
	int n=0;
	
	
	/** Constructor for the offset correction filter using default parameters. <br>
	 * All samples are corrected to a reference value of zero with an update speed of 0.1.
	 * @param source
	 * source for sample
	 */
	public ErrorTermFilter(SampleProvider source) {
		this(source,0.01f);
	}
	


	/**
	 * Constructor for the offset correction filter. <br>
	 * This constrctor takes two parameters for update speed. This is usefull for sensor that have a big offset error.
	 * By giving a high starting speed for offset correction the filter settles quickly to an appropriate offset correction.
	 * By giving a lower end speed the filter is less sensitive to disturbances. 
	 *  By
	 * @param source
	 * Source for sample
	 * @param speed
	 * Begin speed to update offset value
	 * @param endSpeed
	 * End speed to update offset value
	 */
	public ErrorTermFilter(SampleProvider source,  float speed) {
		super(source);
		offset=new float[sampleSize];
		this.speed=speed;
	}


	public void fetchSample(float[] dst, int off) {
		super.fetchSample(dst, off);

		for (int i=0;i<sampleSize;i++) {
		  if (n<SETTLE) {
		    offset[i]=((offset[i]*n+dst[i+off]))/(n+1);
		    dst[i+off]=0;
		  }
		  else {
  			error=dst[i+off]-offset[i];
  			offset[i]=offset[i]*(1.0f-speed)+dst[i+off]*speed;
  			dst[i+off]=error;
  		}
		}
    n++;
	}
}
