package samples;

import actionPilot.*;
import kinematics.*;
import lejos.hardware.Button;
import lejos.hardware.Sound;
import lejos.hardware.motor.EV3LargeRegulatedMotor;
import lejos.hardware.port.MotorPort;
import lejos.utility.Delay;

/**
 * 
 * @author Aswin
 * 
 */
public class Squares {
  static float PI    = (float) Math.PI;
  static float TWOPI = (float) Math.PI * 2;
  ActionPilot  pilot;
  Driver       robot;
  Odometer     odometer;

  /**
   * @param args
   */
  public static void main(String[] args) {
    new Squares();
  }

  private Squares() {
    // Length of the sides of the square
    float side = 800;

    build();
    Sound.beep();
    while (!Button.ENTER.isDown())
      Delay.msDelay(50);

    // define the moves
    Move forward=new Line(side, 0);
    Move backward=new Line(side, -PI );
    Move toLeft=new Line(side, PI / 2);
    Move toRight=new Line(side, -PI / 2);
    Move quarterRotation=new Rotate(PI / 2);
    Move halfRotation=new Rotate(PI);
    
    //Instruct to drive a square while facing north
    pilot.addAction(forward);
    pilot.addAction(toLeft);
    pilot.addAction(backward);
    pilot.addAction(toRight);

    // Instruct to drive a square and turn into driving direction on each corner
    pilot.addAction(forward);
    pilot.addAction(quarterRotation);
    pilot.addAction(toLeft);
    pilot.addAction(quarterRotation);
    pilot.addAction(backward);
    pilot.addAction(quarterRotation);
    pilot.addAction(toRight);
    pilot.addAction(quarterRotation);

    
    //Instruct to drive a square while making a half rotation on each side
    pilot.addAction(new Line(side, 0), halfRotation);
    pilot.addAction(new Line(side, PI / 2), halfRotation);
    pilot.addAction(new Line(side, PI), halfRotation);
    pilot.addAction(new Line(side, -PI / 2), halfRotation);

    while (pilot.isBusy()) {
      Delay.msDelay(100);
      if (Button.ESCAPE.isDown())
        pilot.rest();
    }
    Sound.beep();

    Delay.msDelay(50);
  }

  // define the robot and maximum speed
  private void build() {
    Dynamics maxDynamics = new DynamicsContainer(400, 400, PI / 2, PI / 2);
    ModelBuilder build = new ModelBuilder(3);
    float gear = 24 / 12;
    build.inRadians();
    build.wheelPolarPosition(0, 136, 0, new EV3LargeRegulatedMotor(MotorPort.B), gear);
    build.wheelPolarPosition(1, 136, (PI * 2 / 3), new EV3LargeRegulatedMotor(MotorPort.A), gear);
    build.wheelPolarPosition(2, 136, (PI * 4 / 3), new EV3LargeRegulatedMotor(MotorPort.C), gear);
    robot = new Chassis(build);
    odometer = new Odometer(build);
    pilot = new ActionPilot(robot, odometer, maxDynamics);
    pilot.setBlendMode(true);
  }

}
