package kinematics;

import lejos.robotics.RegulatedMotor;
import lejos.utility.Delay;

/**
 * The chassis represents a robot whitin the driving system. Logically it fits
 * between the pilot, that governs movements, and the motors that make the robot
 * move.
 * <p>
 * The chassis is responsible for translating robot speed into wheel speeds. 
 * All parameters to the chassis are supposed to be in local coordiante system. 
 * (It is not bothered by the robots position.)
 * 
 * @author Aswin
 * 
 */
public class Chassis implements Driver {
	Matrix						kMatrix;
	RegulatedMotor[]	motor;
	int								nWheels;
	Vector temp=new Vector();



	/**
	 * 
	 * @param builder
	 *          A modelBuilder object that can be quiried for a kinematic model
	 */
	public Chassis(ModelBuilder builder) {
		motor = builder.buildMotorArray();
		nWheels = motor.length;
		kMatrix = builder.buildForwardKinematicModel();
	}

	/* (non-Javadoc)
   * @see kinematics.Driver#flt()
   */
	@Override
  public void flt() {
		flt(true);
	}

	/* (non-Javadoc)
   * @see kinematics.Driver#flt(boolean)
   */
	@Override
  public void flt(boolean immediateReturn) {
		for (RegulatedMotor m : motor)
			m.flt();
		if (immediateReturn)
			return;
		waitComplete();
	}

	/* (non-Javadoc)
   * @see kinematics.Driver#isMoving()
   */
	@Override
  public boolean isMoving() {
		for (RegulatedMotor m : motor)
			if (m.isMoving())
				return true;
		return false;
	}

	/* (non-Javadoc)
   * @see kinematics.Driver#stop()
   */
	@Override
  public void stop() {
		stop(true);
	}

	/* (non-Javadoc)
   * @see kinematics.Driver#stop(boolean)
   */
	@Override
  public void stop(boolean immediateReturn) {
		for (RegulatedMotor m : motor)
			m.stop(true);
		if (immediateReturn)
			return;
		waitComplete();
	}

	private void waitComplete() {
		while (isMoving())
			Delay.msDelay(4);
	}

	/* (non-Javadoc)
   * @see kinematics.Driver#move(kinematics.Pose)
   */
	@Override
  public void move(Pose2D speed) {
		temp.set((Vector3D) speed);
		temp.times(kMatrix);
		for (int i = 0; i < nWheels; i++)
			if (motor[i] != null) {
				motor[i].setSpeed((int) temp.get(i));
				if (temp.get(i) < 0)
					motor[i].backward();
				else
					motor[i].forward();
			}
	}
	

}
