package kinematics;

import lejos.robotics.RegulatedMotor;

/**
 * The odometer keeps track of the current pose of a holonomic robot based on the motor encoders (dead reckoning).<p>
 * The odometer runs its own thread to keep up-to-date. 
 * It implements the {@link Pose2D} interface, the current pose can be queried using the methods from this interface.
 * @author Aswin
 *
 */
public class Odometer extends NewPose{
	private final Matrix	iMatrix;
	private final RegulatedMotor[]	motor;
	private int	numberOfWheels=3;
	private  NewPose delta=new NewPose();
	float[] lastTacho=new float[3];
	float[] tacho=new float[3];
	float twoPI=(float)(Math.PI*2);
	float PI=(float)(Math.PI);
 	
	
	public Odometer(ModelBuilder build) {
		iMatrix=build.buildReverseKinematicModel();
		motor=build.buildMotorArray();
		for (int i=0;i<numberOfWheels;i++) 
			tacho[i]=motor[i].getTachoCount();
		PoseTracker tracker = new PoseTracker();
		tracker.setDaemon(true);
		tracker.start();
	}
	

	
	private void updatePose() {
		// get the latest tach values
		for (int i=0;i<numberOfWheels;i++) {
			lastTacho[i]=tacho[i];
			tacho[i]=motor[i].getTachoCount();
		}
		// make a change in position expressed in tacho counts
		delta.setPose(tacho[0]-lastTacho[0],tacho[1]-lastTacho[1],tacho[2]-lastTacho[2]);  	
		// make a change in postion in robot units
    // if (delta.x<2 && delta.y < 2 && delta.h <2) slice=50;
    // else slice=20;
		delta.times(iMatrix);	
		// rotate change in position to world frame;
		delta.rotate(z);
		plus(delta);
	}
	
	
	private class PoseTracker extends Thread {
		
	  private Timing timer;
		
		public void run() {
		  this.setPriority(MAX_PRIORITY);
		  timer=new Timing(.003);
			while (true) {
				timer.delay();
				updatePose();
			}
		}

	}

}
