package samples;

import actionPilot.*;
import kinematics.*;
import lejos.hardware.Button;
import lejos.hardware.Sound;
import lejos.hardware.motor.EV3LargeRegulatedMotor;
import lejos.hardware.port.MotorPort;
import lejos.utility.Delay;

/**
 * Test program for the ActionPilot.
 * cd /home/lejos/programs/
 * jrun -cp Pilot.jar samples/Test
 * 
 * @author Aswin
 * 
 */
public class Test {
  static float      PI      = (float) Math.PI;
  static float      TWOPI   = (float) Math.PI * 2;
  ActionPilot       pilot;
  Driver            robot;
  Odometer          odometer;

  /**
   * @param args
   */
  public static void main(String[] args) {
    new Test();
  }

  private Test() {

    build();
    Sound.beep();
    while (!Button.ENTER.isDown())
      Delay.msDelay(50);
    
//    for (int i=0;i<1;i++) {
//      pilot.addAction(new Line(1200,0));
//      pilot.addAction(new Line(600,PI/2));
//      pilot.addAction(new Line(1200,PI));
//      pilot.addAction(new Line(600,-PI/2));
//   }
//
//    for (int i=0;i<1;i++) {
//       pilot.addAction(new Line(1200,0));
//       pilot.addAction(new Rotate(PI/2) );
//       pilot.addAction(new Line(600,PI/2));
//       pilot.addAction(new Rotate(PI/2) );
//       pilot.addAction(new Line(1200,PI));
//       pilot.addAction(new Rotate(PI/2) );
//       pilot.addAction(new Line(600,-PI/2));
//       pilot.addAction(new Rotate(PI/2) );
//    }
//
//    for (int i=0;i<1;i++) {
//      pilot.addAction(new Line(1200,0),new Rotate(PI));
//      pilot.addAction(new Line(600,PI/2),new Rotate(PI));
//      pilot.addAction(new Line(1200,PI),new Rotate(PI));
//      pilot.addAction(new Line(600,-PI/2),new Rotate(PI));
//   }
    
// pilot.addAction(new MoveTo(new NewPose(600,0,0)));
// Pose2D center=new NewPose(0,300,0);
// pilot.addAction(new Arc(center,PI*2,1) );
// pilot.addAction(new Arc(center,PI*2,1), new Rotate(PI*2) );
// pilot.addAction(new Arc(center,PI*2,1), new Rotate(-PI*2) );
// pilot.addAction(new Arc(center,PI*2,1), new Aim(new NewPose(400,400,0)) );
//   
// pilot.addAction(new MoveTo(Pose2D.ORIGIN));

    
    while (pilot.isBusy()) {
      Delay.msDelay(100);
      //System.out.println(odometer.toString());
      if (Button.ESCAPE.isDown())
        pilot.rest();
    }
    Sound.beep();

    Delay.msDelay(50);
  }

  private void build() {
    Dynamics maxDynamics=new DynamicsContainer(300,300,PI/2,PI/2);
    ModelBuilder build = new ModelBuilder(3);
    float gear = 24 / 12;
    build.inRadians();
    build.wheelPolarPosition(0, 136, 0, new EV3LargeRegulatedMotor(MotorPort.B), gear);
    build.wheelPolarPosition(1, 136, (PI * 2 / 3), new EV3LargeRegulatedMotor(MotorPort.A), gear);
    build.wheelPolarPosition(2, 136, (PI * 4 / 3), new EV3LargeRegulatedMotor(MotorPort.C), gear);
    robot = new Chassis(build);
    odometer = new Odometer(build);
    pilot = new ActionPilot(robot, odometer,maxDynamics);
//    pilot.enableVerbose(ActionPilot.VERBOSE_TIME );
//    pilot.enableVerbose(ActionPilot.VERBOSE_ACTION );
//    pilot.enableVerbose(ActionPilot.VERBOSE_SPEED );
//    pilot.enableVerbose(ActionPilot.VERBOSE_LOOP );
    pilot.setBlendMode(true);
  }

}
